--[[
	create menus with VehicleMenuGUI.createMenu(menuName, prevMenu, title, xpos, ypos, xscale, yscale, buttonSpacing, textScale, font)

	add buttons to each menu with VehicleMenuGUI.addButton(parentMenu, text, funct, args, toggleable, textScale, font)
]]


VehicleMenuGUI = {}
VehicleMenuGUI.menu = {}
VehicleMenuGUI.buttons = {}
VehicleMenuGUI.navButtons = 	{
				["menuOpen"] = 
				{
					["keys"] = {},
					["menu"] = ""
				}, 
				["menuClose"] = {["keys"] = {}},
				["menuPrev"] = {["keys"] = {}},
				["navUp"] = {["keys"] = {}},
				["navDown"] = {["keys"] = {}},
				["select"] = {["keys"] = {}},
				["navBack"] = {["keys"] = {}}
			}
VehicleMenuGUI.activeButton = 1
VehicleMenuGUI.activeMenu = ""
VehicleMenuGUI.statusText = {["text"] = "", ["time"] = 0}

VehicleMenuGUI.titleColor = {["r"] = 25, ["g"] = 25, ["b"] = 112, ["a"] = 225}
VehicleMenuGUI.buttonColor = {["r"] = 65, ["g"] = 105, ["b"] = 225, ["a"] = 200}
VehicleMenuGUI.activeButtonColor = {["r"] = 65, ["g"] = 150, ["b"] = 255, ["a"] = 225}




function VehicleMenuGUI.createMenu(menuName, prevMenu, title, xpos, ypos, xscale, yscale, buttonSpacing, textScale, font, saveButtonPos)
	if(VehicleMenuGUI.menu[menuName] == nil) then
		VehicleMenuGUI.menu[menuName] = {}
		lastOffset = ypos + yscale + buttonSpacing
	end
	
	VehicleMenuGUI.menu[menuName] = {
		["prevMenu"] = prevMenu,
		["title"] = title,
		["xpos"] = xpos,
		["ypos"] = ypos,
		["xscale"] = xscale,
		["yscale"] = yscale,
		["spacing"] = buttonSpacing,
		["lastOffset"] = lastOffset,
		["numButtons"] = 0,
		["textScale"] = textScale,
		["lastButton"] = 1,
		["saveButtonPos"] = saveButtonPos,
		["font"] = font
	}
end

function VehicleMenuGUI.addButton(parentMenu, text, funct, args, toggleable, textScale, font)
	--I tried to make this easy to follow (it probably doesn't help read it, but oh well), but basically,
	--
	if(VehicleMenuGUI.buttons[parentMenu] == nil or VehicleMenuGUI.buttons[parentMenu]["settings"] == nil) then
		VehicleMenuGUI.buttons[parentMenu] = {["settings"] = {}}
	end

	local currButtonNum = #VehicleMenuGUI.buttons[parentMenu].settings + 1
	VehicleMenuGUI.menu[parentMenu].numButtons = currButtonNum
	--in other words, position the button x.spacing under the previous button

	if(toggleable) then
		toggledOn = false
	else
	    toggledOn = nil
	end

	VehicleMenuGUI.buttons[parentMenu].settings[currButtonNum] = {
		["parentMenu"] = parentMenu,
		["text"] = text,
		["funct"] = funct,
		["args"] = args,
		["font"] = font,
		["xpos"] = VehicleMenuGUI.menu[parentMenu].xpos,
		["ypos"] = VehicleMenuGUI.menu[parentMenu].lastOffset,
		["xscale"] = VehicleMenuGUI.menu[parentMenu].xscale,
		["yscale"] = VehicleMenuGUI.menu[parentMenu].yscale,
		["textScale"] = textScale,
		["toggleable"] = toggleable,
		["toggledOn"] = toggledOn
	}

	VehicleMenuGUI.menu[parentMenu]["lastOffset"] = (VehicleMenuGUI.menu[parentMenu].lastOffset + VehicleMenuGUI.menu[parentMenu].yscale + VehicleMenuGUI.menu[parentMenu].spacing)
end

function VehicleMenuGUI.setActiveMenu(menuName)
	VehicleMenuGUI.activeMenu = menuName
	if(VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu] ~= nil) then
		if(VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].saveButtonPos or VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].saveButtonPos == nil) then
			VehicleMenuGUI.activeButton = VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].lastButton
		else
		    VehicleMenuGUI.activeButton = 1
		end
	end
end

function VehicleMenuGUI.updateStatusText(text, time)
	VehicleMenuGUI.statusText = {["text"] = text, ["time"] = time}
end

function VehicleMenuGUI.isActiveButtonToggledOn()
	local activeButton = VehicleMenuGUI.buttons[VehicleMenuGUI.activeMenu].settings[VehicleMenuGUI.activeButton]
	if(activeButton.toggleable) then
		return activeButton.toggledOn
	elseif(activeButton.toggleable == false) then
	    return nil
	end
end

function VehicleMenuGUI.isButtonToggledOn(menu, index)
	local activeButton = VehicleMenuGUI.buttons[menu].settings[index]
	if(activeButton.toggleable) then
		return activeButton.toggledOn
	elseif(activeButton.toggleable == false) then
	    return nil
	end
end

function VehicleMenuGUI.toggleActiveButtonState()
	local activeButton = VehicleMenuGUI.buttons[VehicleMenuGUI.activeMenu].settings[VehicleMenuGUI.activeButton]


	if(VehicleMenuGUI.isActiveButtonToggledOn() and activeButton.toggleable) then
		activeButton.toggledOn = false
	elseif(VehicleMenuGUI.isActiveButtonToggledOn() == false and activeButton.toggleable) then
		activeButton.toggledOn = true
	end
end

function VehicleMenuGUI.toggleButtonState(menu, index)
	local activeButton = VehicleMenuGUI.buttons[menu].settings[index]
	
	if(VehicleMenuGUI.isButtonToggledOn(menu, index) and activeButton.toggleable) then
		activeButton.toggledOn = false
	elseif(VehicleMenuGUI.isButtonToggledOn(menu, index) == false and activeButton.toggleable) then
		activeButton.toggledOn = true
	end
end

--todo
function VehicleMenuGUI.removeButton(parentMenu, index)
	VehicleMenuGUI.buttons[parentMenu].settings[index] = nil
	VehicleMenuGUI.menu[parentMenu]["lastOffset"] = (VehicleMenuGUI.menu[parentMenu].lastOffset - VehicleMenuGUI.menu[parentMenu].yscale - VehicleMenuGUI.menu[parentMenu].spacing)
end

--todo
function VehicleMenuGUI.removeMenu(menuName)
	VehicleMenuGUI.menu[menuName] = nil
end

function VehicleMenuGUI.drawStatusText()
	--this is called every frame from VehicleMenuGUI.tick()
	--this is pretty much a work around because using a while loop causes the game to freeze/crash
	--and using main:Wait(0) causes the text to flash instead of constant
	if(nextDrawTime == nil or prevText ~= VehicleMenuGUI.statusText.text) then
		nextDrawTime = natives.GAMEPLAY.GET_GAME_TIMER() + VehicleMenuGUI.statusText.time
	end

	if(nextDrawTime > natives.GAMEPLAY.GET_GAME_TIMER()) then
		prevText = VehicleMenuGUI.statusText.text
		natives.UI.SET_TEXT_FONT(0)
		natives.UI.SET_TEXT_SCALE(0.55, 0.55)
		natives.UI.SET_TEXT_COLOUR(255, 255, 255, 255)
		natives.UI.SET_TEXT_WRAP(0.0, 1.0)
		natives.UI.SET_TEXT_OUTLINE(1)
		natives.UI.SET_TEXT_CENTRE(true)
		natives.UI.SET_TEXT_DROPSHADOW(1, 0, 0, 255, 255)
		natives.UI.SET_TEXT_EDGE(1, 0, 0, 0, 205)
		natives.UI._SET_TEXT_ENTRY("STRING")
		natives.UI._ADD_TEXT_COMPONENT_STRING(VehicleMenuGUI.statusText.text)
		natives.UI._DRAW_TEXT(0.5, 0.9)
	else
		--clear the text on screen when time is up
		VehicleMenuGUI.statusText = {["text"] = "", ["time"] = 0}
		nextDrawTime = nil
	end
end

function VehicleMenuGUI.drawVehicleMenuGUI()
	--this is pretty much a work around because using a while loop causes the game to freeze/crash
	--and using main:Wait(0) causes the text to flash instead of constant
	if(VehicleMenuGUI.activeMenu ~= "" and VehicleMenuGUI.activeMenu ~= nil) then
		local name = VehicleMenuGUI.activeMenu

		if(VehicleMenuGUI.menu[name] ~= nil) then
			--draw title for one frame
			local xpos = VehicleMenuGUI.menu[name].xpos
			local ypos = VehicleMenuGUI.menu[name].ypos
			local xscale = VehicleMenuGUI.menu[name].xscale
			local yscale = VehicleMenuGUI.menu[name].yscale
			
			local text = VehicleMenuGUI.menu[name].title
			local textScale = VehicleMenuGUI.menu[name].textScale
			local font = VehicleMenuGUI.menu[name].font
			
			local colors = VehicleMenuGUI.titleColor

			VehicleMenuGUI.addButtonText(text, xpos, ypos - 0.007, textScale, font)
			natives.GRAPHICS.DRAW_RECT(xpos, ypos, xscale, yscale, colors.r, colors.g, colors.b, colors.a);

			--done drawing the title for that frame, now draw the buttons
			if(VehicleMenuGUI.buttons[name] ~= nil) then
				for i, v in pairs(VehicleMenuGUI.buttons[name].settings) do
					local buttonSettings = VehicleMenuGUI.buttons[name].settings[i]
					local xpos = buttonSettings.xpos
					local ypos = buttonSettings.ypos
					local xscale = buttonSettings.xscale
					local yscale = buttonSettings.yscale
					
					local font = buttonSettings.font
					local text = buttonSettings.text
					local textScale = buttonSettings.textScale
					local colors = VehicleMenuGUI.buttonColor

					if(buttonSettings.toggledOn == true) then
						text = buttonSettings.text .. " ~b~[ON]"
					elseif(buttonSettings.toggledOn == false) then
						text = buttonSettings.text .. " [OFF]"
					end
					
					if(i == VehicleMenuGUI.activeButton) then
						colors = VehicleMenuGUI.activeButtonColor
					end

					VehicleMenuGUI.addButtonText(text, xpos, ypos, textScale, font)
					natives.GRAPHICS.DRAW_RECT(xpos, ypos, xscale, yscale, colors.r, colors.g, colors.b, colors.a)
				end
			else
				print("Menu ", name, " has no buttons!")
				VehicleMenuGUI.setActiveMenu("")
			end
		else
			print("No menu created with the name: ", VehicleMenuGUI.activeMenu)
			VehicleMenuGUI.setActiveMenu("")
		end
	end
end

function VehicleMenuGUI.addButtonText(text, xpos, ypos, textScale, font)
	if(text ~= nil and xpos ~= nil and ypos ~= nil and textScale ~= nil and font ~= nil) then
		natives.UI.SET_TEXT_FONT(font)
		natives.UI.SET_TEXT_SCALE(0.0, textScale)
		natives.UI.SET_TEXT_COLOUR(255, 255, 255, 255)
		natives.UI.SET_TEXT_CENTRE(true)
		natives.UI.SET_TEXT_OUTLINE(1)
		natives.UI.SET_TEXT_DROPSHADOW(1, 0, 0, 255, 255)
		natives.UI.SET_TEXT_EDGE(0, 0, 0, 0, 0)
		natives.UI._SET_TEXT_ENTRY("STRING")
		natives.UI._ADD_TEXT_COMPONENT_STRING(text)
		natives.UI._DRAW_TEXT(xpos, ypos - 0.0125)
	else
		print("text: ", text, ", xpos: ", xpos, ", ypos: ", ypos, ", textScale: ", textScale, ", font: ", font)
	    print("There seems to be a nil value when trying to add text to button (see above)")
	    --VehicleMenuGUI.setActiveMenu("")
	end
end

function VehicleMenuGUI.runButtonFunct(currButton)
	--getting the type of a table broke in one of the updates so this is a workaround
	if(type(currButton.funct) == "function" and currButton.args ~= nil and type(currButton.args) ~= "string" and type(currButton.args) ~= "number" and type(currButton.args) ~= "boolean") then
		currButton.funct(currButton.args[1], currButton.args[2], currButton.args[3], currButton.args[4], currButton.args[5], currButton.args[6], currButton.args[7], currButton.args[8], currButton.args[9], currButton.args[10], currButton.args[11], currButton.args[12], currButton.args[13], currButton.args[14], currButton.args[15], currButton.args[16], currButton.args[17], currButton.args[18], currButton.args[19], currButton.args[20])
		return true
	end

	if(type(currButton.funct) == "function" and type(currButton.args) == "string" or type(currButton.args) == "number" or type(currButton.args) == "boolean") then
		currButton.funct(currButton.args)
		return true
	end

	if(currButton.funct ~= nil and currButton.args == nil  ) then
		currButton.funct()
		return true
	end

	if(currButton.funct == nil) then
		print("Unabled to run the function")
		return false
	end
end

--this needs to be called every frame
function VehicleMenuGUI.tick()
	--this is pretty much a work around because using a while loop causes the game to freeze/crash
	--and using main:Wait(0) causes the text to flash instead of constant
	--this is called with the main:Run() loop
	VehicleMenuGUI.drawStatusText()
	VehicleMenuGUI.drawVehicleMenuGUI()

	--on menu open key press
	for i, key in pairs(VehicleMenuGUI.navButtons.menuOpen.keys) do
		if(IsKeyDown(key)) then
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "QUIT_WHOOSH", "HUD_MINI_GAME_SOUNDSET", false)
			if(VehicleMenuGUI.activeMenu == VehicleMenuGUI.navButtons.menuOpen.menu) then
				--if menu that is trying to be drawn is already drawn, go to the previous menu that is set for that menu 
				VehicleMenuGUI.setActiveMenu(VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].prevMenu)
			else
				--if it isn't already drawn, draw as normal
			    VehicleMenuGUI.setActiveMenu(VehicleMenuGUI.navButtons.menuOpen.menu)
			    VehicleMenuGUI.currentSelection = 1
			end
		end
	end

	--on menu close key press
	for i, key in pairs(VehicleMenuGUI.navButtons.menuClose.keys) do
		if(IsKeyDown(key)) then
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)
			VehicleMenuGUI.setActiveMenu("")
		end
	end

	--if the navUp key is pressed, move up (down numerically) in the menu
	for k, key in pairs(VehicleMenuGUI.navButtons.navUp.keys) do
		if(IsKeyDown(key) and VehicleMenuGUI.activeMenu ~= "") then
			VehicleMenuGUI.activeButton = VehicleMenuGUI.activeButton - 1
			VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].lastButton = VehicleMenuGUI.activeButton
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)
			
			--loop selection to the last button if <= 0
			if(VehicleMenuGUI.activeButton <= 0) then
				VehicleMenuGUI.activeButton = VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].numButtons
				VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].lastButton = VehicleMenuGUI.activeButton
			end
		end
	end

	--if the navDown key is pressed, move down (up numerically) in the menu
	for k, key in pairs(VehicleMenuGUI.navButtons.navDown.keys) do
		if(IsKeyDown(key) and VehicleMenuGUI.activeMenu ~= "") then
			VehicleMenuGUI.activeButton = VehicleMenuGUI.activeButton + 1
			VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].lastButton = VehicleMenuGUI.activeButton
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)

			--loop selection to the first button if > the last button
			if(VehicleMenuGUI.activeButton > VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].numButtons) then
				VehicleMenuGUI.activeButton = 1
				VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].lastButton = VehicleMenuGUI.activeButton
			end
		end
	end

	--if the select key is pressed, call the function the button was pressed on
	for k, key in pairs(VehicleMenuGUI.navButtons.select.keys) do
		if(IsKeyDown(key) and VehicleMenuGUI.activeMenu ~= "") then
			
			local currButton = VehicleMenuGUI.buttons[VehicleMenuGUI.activeMenu].settings[VehicleMenuGUI.activeButton]
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)

			if(VehicleMenuGUI.runButtonFunct(currButton)) then
				if(currButton.toggleable) then
					VehicleMenuGUI.toggleActiveButtonState()
				end
			end
		end
	end

	--if the navBack key is pressed, draw the prevMenu
	for k, key in pairs(VehicleMenuGUI.navButtons.navBack.keys) do
		if(IsKeyDown(key) and VehicleMenuGUI.activeMenu ~= "") then
			VehicleMenuGUI.setActiveMenu(VehicleMenuGUI.menu[VehicleMenuGUI.activeMenu].prevMenu)
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false);
		end
	end
end

function VehicleMenuGUI:OnError()
	--self:Kill()
	--addon.Load("RENAMEME")
end